package muyu.data.ranges.common.domain.dataSoutce;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.muyu.system.domain.DataAccess;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * 线程池配置
 */
@Data
@Component
@Configuration
@Log4j2
public class DataSourceConfig {

    private static HashMap<Long,DruidDataSource> hashMap = new HashMap<>();
    /**
     * 添加线程池
     * 线程池初始化
     */
    public static void index(DataAccess dataAccess) throws SQLException {
        //主机地址
        String host = dataAccess.getHost();
        //端口号
        String port = dataAccess.getPort();
        //数据库名
        String databaseName = dataAccess.getDatabaseName();
        // url
        String url = String.format("jdbc:mysql://"+ host+":"+port+":"+databaseName);
        log.info("集成化的url:{}",url);
        DruidDataSource dataSource = new DruidDataSource();
        //初始化链接池数量
        dataSource.setInitialSize(dataAccess.getInitNum().intValue());
        //最大线程池
        dataSource.setMaxActive(dataAccess.getMaxNum().intValue());
        //最大等待时间
        dataSource.setMaxWait(dataAccess.getMaxWaitTime());
        //配置数据库的基本信息
        dataSource.setDriverClassName("com.mysql/jdbc.Driver");
        //数据库连接池
        dataSource.setUrl("jdbc:mysql://"+host+":"+port+":"+databaseName);
        //数据库用户名
        dataSource.setUsername(dataAccess.getUsername());
        //数据库密码
        dataSource.setPassword(dataAccess.getPassword());
        try {
            dataSource.init();// 初始化
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        hashMap.put(dataAccess.getId(),dataSource);
        SpringHashMap(hashMap);
    }

    /**
     * 取出线程池
     */
    public static Connection getConnection(Long key){
        DruidDataSource dataSource = hashMap.get(key);
        try {
            DruidPooledConnection connection = dataSource.getConnection();
            return connection;
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * 归还线程池
     */
    public static void returnConnection(Connection connection){
        try {
            connection.close();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * 连接线程池
     * 验证线程池
     */
    public static void verify(Long key) throws SQLException {
        DruidDataSource dataSource = hashMap.get(key);
        // 获取连接
        DruidPooledConnection connection = dataSource.getConnection();
        log.info("连接上线程池：{}",connection);
    }

    @Bean
    public static HashMap<Long,DruidDataSource> SpringHashMap(HashMap<Long, DruidDataSource> hashMap) {
        return hashMap;
    }
}
