package com.muyu.efition.clinet.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.edition.domain.RuleEngine;
import com.muyu.edition.domain.vo.UnitVo;
import com.muyu.goods.edition.remote.RemoteDataManagerService;
import com.muyu.system.domain.DataAccess;
import com.muyu.system.domain.vo.FrimaryVo;
import com.muyu.system.remote.RemoteSystemManageService;
import com.muyu.system.remote.factory.DataMangSystemFactory;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 初始化加载
 */
@Log4j2
@Component
@Configuration
public class AccessConfigRunner implements ApplicationRunner {
    @Autowired
    private RemoteSystemManageService remoteSystemManageService;

    @Override
    public void run(ApplicationArguments args) throws Exception {
        //远程调用
        Result<TableDataInfo<DataAccess>> listed = remoteSystemManageService.list(null);
        //判断非空
        if (listed.getData().getRows() == null || listed.getData().getRows().isEmpty()){
            log.error("数据为空");
            return;
        }
        //获取数据源集合
        List<DataAccess> listResultData = listed.getData().getRows();
        //存入map
        //创建map
        HashMap<Long, DataAccess> frimaryVoHashMap = new HashMap<>();
        //遍历循环存入map
        listResultData.forEach(res -> {
            frimaryVoHashMap.put(res.getId(),res);
        });
        HashMap<String, UnitVo> hashMap = new HashMap<>();
        //遍历添加到hashMap
        for (DataAccess listResultDatum : listResultData) {
            if (listResultDatum.getType() == "MySql"){
                //主机地址
                String host = listResultDatum.getHost();
                //端口号
                String port = listResultDatum.getPort();
                //数据库名
                String databaseName = listResultDatum.getDatabaseName();
                // url
                String url = String.format("jdbc:mysql://"+ host+":"+port+":"+databaseName);
                log.info("集成化的url:{}",url);
                //连接对象 url/用户名/密码
                Connection connection = DriverManager.getConnection(url,listResultDatum.getUsername(),listResultDatum.getPassword());
                log.info("连接对象：{}",connection);
                DruidDataSource dataSource = new DruidDataSource();
                //初始化链接池数量
                dataSource.setInitialSize(listResultDatum.getInitNum().intValue());
                //最大线程池
                dataSource.setMaxActive(listResultDatum.getMaxNum().intValue());
                //最大等待时间
                dataSource.setMaxWait(listResultDatum.getMaxWaitTime());
                //配置数据库的基本信息
                dataSource.setDriverClassName("com.mysql/jdbc.Driver");
                //数据库连接池
                dataSource.setUrl("jdbc:mysql://"+
                        listResultDatum.getHost()+":"+listResultDatum.getPort()+":"+listResultDatum.getDatabaseName());
                //数据库用户名
                dataSource.setUsername(listResultDatum.getUsername());
                //数据库密码
                dataSource.setPassword(listResultDatum.getPassword());
                log.info("获取连接池对象：{}",dataSource);
                //封装对象
                //SuperBuilder方法
                UnitVo build = UnitVo.builder()
                        .connection(connection)
                        .dataSource(dataSource)
                        .build();
                //存入map
                hashMap.put(host+"_"+port+"_"+databaseName,build);
                //查看连接池
                DataAccess dataAccess = listResultData.get(0);
                    UnitVo unitVo = hashMap.get(dataAccess.getHost() + "_" + dataAccess.getPort() + "_" + dataAccess.getDatabaseName());
                    log.info("存在的连接池有:--{}",unitVo);
                //取出连接池

                //返回连接池

            }
        }
        //将对象传入Spring
        SpringHashMap(hashMap);
    }

    @Bean
    public HashMap<String,UnitVo> SpringHashMap(HashMap<String,UnitVo> hashMap) {
        return hashMap;
    }



}
