package com.muyu.edition.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.edition.domain.Edition;
import com.muyu.edition.service.IEditionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 规则引擎版本Controller
 *
 * @author muyu
 * @date 2024-05-06
 */
@RestController
@RequestMapping("/edition")
public class EditionController extends BaseController {
    @Autowired
    private IEditionService editionService;

    /**
     * 查询规则引擎版本列表
     */
    @RequiresPermissions("goods:edition:list")
    @GetMapping("/list")
    public Result<TableDataInfo<Edition>> list() {
        startPage();
        List<Edition> list = editionService.selectEditionList();
        return getDataTable(list);
    }

    /**
     * 获取相应的规则引擎版本
     *
     * @param ruleId
     * @return
     */
    @PostMapping("/selectListRuleId/{ruleId}")
    public Result<List<Edition>> selectListRuleId(@PathVariable Long ruleId) {
        return success(editionService.selectListRuleId(ruleId));
    }

    /**
     * 导出规则引擎版本列表
     */
    @RequiresPermissions("goods:edition:export")
    @Log(title = "规则引擎版本", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Edition edition) {
        List<Edition> list = editionService.selectEditionList();
        ExcelUtil<Edition> util = new ExcelUtil<Edition>(Edition.class);
        util.exportExcel(response, list, "规则引擎版本数据");
    }

    /**
     * 获取规则引擎版本详细信息
     */
    @RequiresPermissions("goods:edition:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id) {
        return success(editionService.selectEditionById(id));
    }

    /**
     * 新增规则引擎版本
     */
    @RequiresPermissions("goods:edition:add")
    @Log(title = "规则引擎版本", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody Edition edition) {
        return toAjax(editionService.insertEdition(edition));
    }

    /**
     * 修改规则引擎版本
     */
    @RequiresPermissions("goods:edition:edit")
    @Log(title = "规则引擎版本", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody Edition edition) {
        return toAjax(editionService.updateEdition(edition));
    }

    /**
     * 删除规则引擎版本
     */
    @RequiresPermissions("goods:edition:remove")
    @Log(title = "规则引擎版本", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids) {
        return toAjax(editionService.deleteEditionByIds(ids));
    }

}
