package com.muyu.edition.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.edition.domain.RuleEngine;
import com.muyu.edition.service.IRuleEngineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 规则引擎Controller
 *
 * @author muyu
 * @date 2024-05-02
 */
@RestController
@RequestMapping("/engine")
public class RuleEngineController extends BaseController {
    @Autowired
    private IRuleEngineService ruleEngineService;

    /**
     * 查询规则引擎列表
     */
    @RequiresPermissions("goods:engine:list")
    @GetMapping("/list")
    public Result<TableDataInfo<RuleEngine>> list(RuleEngine ruleEngine) {
        List<RuleEngine> list = ruleEngineService.selectRuleEngineList(ruleEngine);
        return getDataTable(list);
    }

    /**
     * 导出规则引擎列表
     */
    @RequiresPermissions("goods:engine:export")
    @Log(title = "规则引擎", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RuleEngine ruleEngine) {
        List<RuleEngine> list = ruleEngineService.selectRuleEngineList(ruleEngine);
        ExcelUtil<RuleEngine> util = new ExcelUtil<RuleEngine>(RuleEngine.class);
        util.exportExcel(response, list, "规则引擎数据");
    }

    /**
     * 获取规则引擎详细信息
     */
    @RequiresPermissions("goods:engine:query")
    @GetMapping(value = "/{ruleId}")
    public Result getInfo(@PathVariable("ruleId") Long ruleId) {
        return success(ruleEngineService.selectRuleEngineByRuleId(ruleId));
    }

    @PostMapping("selectRuleEngineOne/{ruleId}")
    public Result selectRuleEngineOne(@PathVariable Long ruleId) {
        return success(ruleEngineService.selectRuleEngineOne(ruleId));
    }

    /**
     * 新增规则引擎
     */
    @RequiresPermissions("goods:engine:add")
    @Log(title = "规则引擎", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody RuleEngine ruleEngine) {
        return toAjax(ruleEngineService.insertRuleEngine(ruleEngine));
    }

    /**
     * 修改规则引擎
     */
    @RequiresPermissions("goods:engine:edit")
    @Log(title = "规则引擎", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody RuleEngine ruleEngine) {
        return toAjax(ruleEngineService.updateRuleEngine(ruleEngine));
    }

    /**
     * 删除规则引擎
     */
    @RequiresPermissions("goods:engine:remove")
    @Log(title = "规则引擎", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ruleIds}")
    public Result remove(@PathVariable Long[] ruleIds) {
        return toAjax(ruleEngineService.deleteRuleEngineByRuleIds(ruleIds));
    }

    /**
     * 规则描述
     *
     * @param ruleId
     * @return
     */
    @PostMapping("/description/{ruleId}")
    public Result<String> description(@PathVariable Integer ruleId) {
        return success(ruleEngineService.description(ruleId));
    }

    @PutMapping("/updateRuleIsActivate")
    public Result updateRuleIsActivate(@RequestBody RuleEngine ruleEngine) {
        ruleEngineService.updateRuleIsActivate(ruleEngine);
        return success("改变激活状态");
    }

    @PutMapping("/updateRuleStatus")
    public Result updateRuleStatus(@RequestBody RuleEngine ruleEngine) {
        ruleEngineService.updateRuleStatus(ruleEngine);
        return success("改变规则状态");
    }

    @PostMapping("/spliceNameToCode")
    public Result spliceNameToCode(@RequestParam String name, @RequestParam String code, @RequestParam Integer level) {
        return ruleEngineService.spliceNameToCode(name, code, level);
    }

}
