package com.muyu.edition.service;

import com.muyu.edition.domain.Config;
import com.muyu.edition.domain.model.TextData;
import com.muyu.edition.domain.resp.EngineConfigResp;

import java.util.List;

/**
 * 引擎Service接口
 *
 * @author muyu
 * @date 2024-05-04
 */
public interface IConfigService {
    /**
     * 查询引擎
     *
     * @param id 引擎主键
     * @return 引擎
     */
    public Config selectConfigById(Long id);

    /**
     * 查询引擎列表
     *
     * @param config 引擎
     * @return 引擎集合
     */
    public List<Config> selectConfigList(Config config);

    /**
     * 查询引擎列表
     *
     * @param ruleId
     * @return
     */
    List<Config> listConfigEs(Long ruleId);

    /**
     * 新增引擎
     *
     * @param config 引擎
     * @return 结果
     */
    public int insertConfig(Config config);

    /**
     * 修改引擎
     *
     * @param config 引擎
     * @return 结果
     */
    public int updateConfig(Config config);

    /**
     * 批量删除引擎
     *
     * @param ids 需要删除的引擎主键集合
     * @return 结果
     */
    public int deleteConfigByIds(Long[] ids);

    /**
     * 删除引擎信息
     *
     * @param id 引擎主键
     * @return 结果
     */
    public int deleteConfigById(Long id);

    Object testData(TextData textData);

    /**
     * 通过引擎作用域获取引擎配置作用域信息
     *
     * @param id
     * @return
     */
    EngineConfigResp getScopeInfo(Integer id);

    List<Config> queryConfigInfo(Long ruleId);
}
