package com.muyu.edition.service.impl;

import com.muyu.common.core.exception.ServiceException;
import com.muyu.edition.constant.ConfigCodeConstants;
import com.muyu.edition.constant.RuleOperationConstants;
import com.muyu.edition.domain.Config;
import com.muyu.edition.domain.model.TextData;
import com.muyu.edition.domain.resp.EngineConfigResp;
import com.muyu.edition.dymamicLoad.DynamicLoader;
import com.muyu.edition.mapper.ConfigMapper;
import com.muyu.edition.service.IConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.Method;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 引擎Service业务层处理
 *
 * @author muyu
 * @date 2024-05-04
 */
@Service
public class ConfigServiceImpl implements IConfigService {
    @Autowired
    private ConfigMapper configMapper;

    /**
     * 查询引擎
     *
     * @param id 引擎主键
     * @return 引擎
     */
    @Override
    public Config selectConfigById(Long id) {
        return configMapper.selectConfigById(id);
    }

    /**
     * 查询引擎列表
     *
     * @param config 引擎
     * @return 引擎
     */
    @Override
    public List<Config> selectConfigList(Config config) {
        return configMapper.selectConfigList(config);
    }

    /**
     * y引擎列表
     *
     * @param ruleId
     * @return
     */
    @Override
    public List<Config> listConfigEs(Long ruleId) {
        return configMapper.listConfigEs(ruleId);
    }

    /**
     * 新增引擎
     *
     * @param config 引擎
     * @return 结果
     */
    @Override
    public int insertConfig(Config config) {
        return configMapper.insertConfig(config);
    }

    /**
     * 修改引擎
     *
     * @param config 引擎
     * @return 结果
     */
    @Override
    public int updateConfig(Config config) {
        return configMapper.updateConfig(config);
    }

    /**
     * 批量删除引擎
     *
     * @param ids 需要删除的引擎主键
     * @return 结果
     */
    @Override
    public int deleteConfigByIds(Long[] ids) {
        return configMapper.deleteConfigByIds(ids);
    }

    /**
     * 删除引擎信息
     *
     * @param id 引擎主键
     * @return 结果
     */
    @Override
    public int deleteConfigById(Long id) {
        return configMapper.deleteConfigById(id);
    }

    @Override
    public Object testData(TextData textData) {
        Object invoke = null;
        try {
            Config config = configMapper.selectConfigById(textData.getId());
            String content = config.getRuleContent().replaceAll("\r\n", "");
            // 对source进行编译生成class文件存放在Map中这里用bytecode接收，
            Map<String, byte[]> bytecode = DynamicLoader.compile(RuleOperationConstants.CLASS_NAME + RuleOperationConstants.FILE_SUFFIX, content);

            // 加载class文件到虚拟机中，然后通过反射执行
            @SuppressWarnings("resource")
            DynamicLoader.MemoryClassLoader classLoader = new DynamicLoader.MemoryClassLoader(
                    bytecode);
            Class<?> clazz = classLoader.loadClass(RuleOperationConstants.CLASS_NAME);

            // 调用ruleTest方法
            Method mainMethod = clazz.getDeclaredMethod(RuleOperationConstants.METHOD_NAME, List.class);
            invoke = mainMethod.invoke(null, textData.getList());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException("测试失败");
        }
        return invoke;
    }

    /**
     * 通过引擎作用域获取引擎配置作用域信息
     *
     * @param id
     * @return
     */
    @Override
    public EngineConfigResp getScopeInfo(Integer id) {
        String scope = ConfigCodeConstants.CONFIG_FILE_NAME_ARRAY[id];
        String type = ConfigCodeConstants.CONFIG_FILE_TYPE_ARRAY[id];
        String path = ConfigCodeConstants.BASE_FILE_PATH + scope;
        String code = null;
        try {
            code = Files.readString(Paths.get(path));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return EngineConfigResp.builder()
                .type(type)
                .name(scope)
                .code(code)
                .build();
    }

    @Override
    public List<Config> queryConfigInfo(Long ruleId) {
        return selectConfigList(null).stream().filter(c -> c.getRuleId().equals(ruleId)).collect(Collectors.toList())
                ;
    }
}
