package com.muyu.edition.service.impl;

import com.muyu.edition.domain.Edition;
import com.muyu.edition.mapper.EditionMapper;
import com.muyu.edition.service.IEditionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 规则引擎版本Service业务层处理
 *
 * @author muyu
 * @date 2024-05-06
 */
@Service
public class EditionServiceImpl implements IEditionService {
    @Autowired
    private EditionMapper editionMapper;

    /**
     * 查询规则引擎版本
     *
     * @param id 规则引擎版本主键
     * @return 规则引擎版本
     */
    @Override
    public Edition selectEditionById(Long id) {
        return editionMapper.selectEditionById(id);
    }

    /**
     * 获取相应的规则引擎版本
     *
     * @param ruleId
     * @return
     */
    @Override
    public List<Edition> selectListRuleId(Long ruleId) {
        return selectEditionList().stream().filter(c -> c.getRuleId().equals(ruleId)).collect(Collectors.toList());
    }

    /**
     * 查询规则引擎版本列表
     *
     * @return 规则引擎版本
     */
    @Override
    public List<Edition> selectEditionList() {
        return editionMapper.selectEditionList();
    }

    /**
     * 新增规则引擎版本
     *
     * @param edition 规则引擎版本
     * @return 结果
     */
    @Override
    public int insertEdition(Edition edition) {
        return editionMapper.insertEdition(edition);
    }

    /**
     * 修改规则引擎版本
     *
     * @param edition 规则引擎版本
     * @return 结果
     */
    @Override
    public int updateEdition(Edition edition) {
        return editionMapper.updateEdition(edition);
    }

    /**
     * 批量删除规则引擎版本
     *
     * @param ids 需要删除的规则引擎版本主键
     * @return 结果
     */
    @Override
    public int deleteEditionByIds(Long[] ids) {
        return editionMapper.deleteEditionByIds(ids);
    }

    /**
     * 删除规则引擎版本信息
     *
     * @param id 规则引擎版本主键
     * @return 结果
     */
    @Override
    public int deleteEditionById(Long id) {
        return editionMapper.deleteEditionById(id);
    }
}
