package com.muyu.edition.service.impl;

import com.muyu.common.core.domain.Result;
import com.muyu.edition.constant.ConfigCodeConstants;
import com.muyu.edition.domain.RuleEngine;
import com.muyu.edition.domain.req.RuleEditionReq;
import com.muyu.edition.mapper.RuleEngineMapper;
import com.muyu.edition.service.IRuleEngineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 规则引擎Service业务层处理
 *
 * @author muyu
 * @date 2024-05-02
 */
@Service
public class RuleEngineServiceImpl implements IRuleEngineService {
    @Autowired
    private RuleEngineMapper ruleEngineMapper;

    /**
     * 查询规则引擎
     *
     * @param ruleId 规则引擎主键
     * @return 规则引擎
     */
    @Override
    public RuleEngine selectRuleEngineByRuleId(Long ruleId) {
        return ruleEngineMapper.selectRuleEngineByRuleId(ruleId);
    }

    /**
     * 规则引擎详情
     *
     * @param ruleId
     * @return
     */
    @Override
    public RuleEngine selectRuleEngineOne(Long ruleId) {
        List<RuleEngine> ruleEngines = selectRuleEngineList(null);
        List<RuleEngine> collect = ruleEngines.stream().filter(c -> c.getRuleId().equals(ruleId)).collect(Collectors.toList());
        return collect.get(0);
    }

    /**
     * 查询规则引擎列表
     *
     * @param ruleEngine 规则引擎
     * @return 规则引擎
     */
    @Override
    public List<RuleEngine> selectRuleEngineList(RuleEngine ruleEngine) {
        return ruleEngineMapper.selectRuleEngineList(ruleEngine);
    }

    /**
     * 新增规则引擎
     *
     * @param ruleEngine 规则引擎
     * @return 结果
     */
    @Override
    public int insertRuleEngine(RuleEngine ruleEngine) {
        return ruleEngineMapper.insertRuleEngine(ruleEngine);
    }

    /**
     * 修改规则引擎
     *
     * @param ruleEngine 规则引擎
     * @return 结果
     */
    @Override
    public int updateRuleEngine(RuleEngine ruleEngine) {
        return ruleEngineMapper.updateRuleEngine(ruleEngine);
    }

    /**
     * 批量删除规则引擎
     *
     * @param ruleIds 需要删除的规则引擎主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineByRuleIds(Long[] ruleIds) {
        return ruleEngineMapper.deleteRuleEngineByRuleIds(ruleIds);
    }

    /**
     * 删除规则引擎信息
     *
     * @param ruleId 规则引擎主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineByRuleId(Long ruleId) {
        return ruleEngineMapper.deleteRuleEngineByRuleId(ruleId);
    }

    @Override
    public String description(Integer ruleId) {
        return ruleEngineMapper.description(ruleId);
    }

    @Override
    public void updateRuleIsActivate(RuleEngine ruleEngine) {
        if (ruleEngine.getRuleIsActivate().equals("Y")) {
            ruleEngine.setRuleIsActivate("N");
        } else {
            ruleEngine.setRuleIsActivate("Y");
        }
        ruleEngineMapper.updateRuleIsActivate(ruleEngine.getRuleId(), ruleEngine.getRuleIsActivate());
    }

    @Override
    public void updateRuleStatus(RuleEngine ruleEngine) {
        if (ruleEngine.getRuleStatus().equals("Y")) {
            ruleEngine.setRuleStatus("N");
        } else {
            ruleEngine.setRuleStatus("Y");
        }
        ruleEngineMapper.updateRuleStatus(ruleEngine.getRuleId(), ruleEngine.getRuleStatus());
    }

    @Override
    public Result spliceNameToCode(String name, String code, Integer level) {
        List<String> stringList = List.of(new String[]{"questEngine", "dataSetEngine", "recordsEngine", "fieldEngine"});
        RuleEditionReq ruleEditionReq = new RuleEditionReq();
        String val = name + "_" + code;
        String scope = ConfigCodeConstants.CONFIG_FILE_NAME_CODE[0];
        String path = ConfigCodeConstants.BASE_FILE_PATH + scope;
        String cod = null;
        try {
            cod = Files.readString(Paths.get(path));
            String s1 = Pattern.compile("this.form.name").matcher(cod).replaceAll(name);
            String s2 = Pattern.compile("this.form.versionCode").matcher(s1).replaceAll(code);
            String s3 = Pattern.compile("parentClass").matcher(s2).replaceAll(stringList.get(level - 1));
            ruleEditionReq.setVal(val);
            ruleEditionReq.setCode(s3);
            System.out.println(cod);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return Result.success(ruleEditionReq);
    }
}
