package com.muyu.system.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.system.domain.*;
import com.muyu.system.domain.table.TableShow;
import com.muyu.system.domain.table.ToTalNum;
import com.muyu.system.domain.vo.DeptVO;
import com.muyu.system.domain.vo.FrimaryVo;
import com.muyu.system.domain.vo.SysUserVo;
import com.muyu.system.domain.vo.TableVo;
import com.muyu.system.service.AccreditService;
import com.muyu.system.service.SysDeptService;
import com.muyu.system.service.SysUserService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

@RestController
@RequestMapping("accredit")
@Log4j2
public class AccreditController extends BaseController {
    @Autowired
    private AccreditService service;
    //部门信息
    @Autowired
    private SysDeptService deptService;
    @Autowired
    private SysUserService userService;

    /**
     * 获取表名
     *
     * @return
     */
    @PostMapping("/tableNameList/{databaseName}")
    public Result<List<TableList>> tableNameList(@PathVariable String databaseName) {
        List<TableList> list = service.tableNameList(databaseName);
        return Result.success(list);
    }

    /**
     * 获取表名
     *
     * @return
     */
    @GetMapping("/tableNameList2")
    public Result<List<String>> tableNameList2() throws SQLException {
        List<String> list = service.tableNameList2();
        return Result.success(list);
    }

    /**
     * 获取表结构
     *
     * @return
     * @throws SQLException
     */
    @GetMapping("/listSqlJdbc")
    public Result listSqlJdbc(@RequestBody Connection connection, String tableName) throws SQLException {
        return service.listSqlJdbc(connection, tableName);
    }

    /**
     * 数据连接
     *
     * @param dataAccess
     * @return
     */
    @PostMapping("/selJDBCStream")
    public Result selJDBCStream(@RequestBody DataAccess dataAccess) {
        Integer id = service.selJDBCStream(dataAccess);
        if (id > 0) {
            return success("连接成功");
        } else {
            return error("连接失败");
        }
    }

    //    测试
    @PostMapping("/selJDBCStream2")
    public Result selJDBCStream2(@RequestBody DataAccess dataAccess) {
        Integer id = service.selJDBCStream2(dataAccess);
        if (id > 0) {
            return success("连接成功");
        } else {
            return error("连接失败");
        }
    }

    /**
     * 获取表详情
     *
     * @param tableName
     * @return
     */
    @PostMapping("/selectTableXml/{tableName}")
    public Result<List<TableVo>> selectTableXml(@PathVariable String tableName) {
        List<TableVo> list = service.selectTableXml(tableName);
        return Result.success(list);
    }

    /**
     * 查询部门
     *
     * @return
     */
    @GetMapping("/listDeptVo")
    public Result list() {
        List<DeptVO> depts = deptService.selectDeptListVo();
        return success(depts);
    }

    /**
     * 查询用户
     *
     * @return
     */
    @GetMapping("/listSelectSysUser")
    public Result<List<SysUserVo>> listSelectSysUser() {
        List<SysUserVo> list = userService.listSelectSysUser();
        return success(list);
    }

    /**
     * 获取表结构
     *
     * @return
     */
    @PostMapping("/listListstructure")
    public Result<List<ListStructure>> listListstructure() {
        List<ListStructure> list = service.listListstructure();
        return success(list);
    }

    /**
     * 根据名称查询表框架
     *
     * @param name
     * @return
     */
    @PostMapping("/selNameTableList")
    public Result<TableList> selNameTableList(@RequestParam String databaseName, @RequestParam String name) {
        TableList tableList = service.selNameTableList(databaseName, name);
        return success(tableList);
    }

    /**
     * 根据名称获取表结构
     *
     * @param databaseName
     * @return
     */
    @PostMapping("/selNameListStructure")
    public Result<List<ListStructure>> selNameListStructure(@RequestParam String tableName, @RequestParam String databaseName) {
        List<ListStructure> list = service.selNameListStructure(tableName, databaseName);
        return success(list);
    }

    /**
     * 修改状态/添加字典
     *
     * @param listStructure
     * @return
     */
    @PostMapping("/updateListStructure")
    public Result updateListStructure(@RequestBody ListStructure listStructure) {
        Integer id = service.updateListStructure(listStructure);
        if (id > 0) {
            return success("修改完成");
        } else {
            return error("修改有异常请检查");
        }
    }

    /**
     * 查看父表
     *
     * @return
     */
    @PostMapping("/selectFrimary")
    public Result<List<FrimaryVo>> selectFrimary() {
        List<FrimaryVo> list = service.selectFrimary();
        return success(list);
    }

    /**
     * 获取部门
     *
     * @param tableName
     * @param databaseName
     * @return
     */
    @PostMapping("/listMiddle")
    public Result<List<Middle>> listMiddle(@RequestParam String tableName, @RequestParam String databaseName) {
        List<Middle> middleList = service.listMiddle(tableName, databaseName);
        return success(middleList);
    }

    /**
     * 获取部门2
     *
     * @param databaseName
     * @return
     */
    @PostMapping("/listMiddle2")
    public Result<List<Middle>> listMiddle2(@RequestParam String databaseName) {
        List<Middle> list = service.listMiddle2(databaseName);
        return success(list);
    }

    /**
     * 获取用户
     *
     * @param tableName
     * @param databaseName
     * @return
     */
    @PostMapping("/listUserMiddle")
    public Result<List<UserMiddle>> listUserMiddle(@RequestParam String tableName, @RequestParam String databaseName) {
        List<UserMiddle> list = service.listUserMiddle(tableName, databaseName);
        return success(list);
    }

    /**
     * 获取用户2
     *
     * @param databaseName
     * @return
     */
    @PostMapping("/listUserMiddle2")
    public Result<List<UserMiddle>> listUserMiddle2(@RequestParam String databaseName) {
        List<UserMiddle> list = service.listUserMiddle2(databaseName);
        return success(list);
    }

    /**
     * 部门授权
     *
     * @param mid
     * @param statusId
     * @return
     */
    @PostMapping("/updateMiddle")
    public Result updateMiddle(@RequestParam Integer mid, @RequestParam String statusId) {
        Integer id = service.updateMiddle(mid, statusId);
        if (id > 0) {
            return success("修改完成");
        } else {
            return error("异常");
        }
    }

    /**
     * 用户授权
     *
     * @param mid
     * @param statusId
     * @return
     */
    @PostMapping("/updateUserMiddle")
    public Result updateUserMiddle(@RequestParam Integer mid, @RequestParam String statusId) {
        Integer id = service.updateUserMiddle(mid, statusId);
        if (id > 0) {
            return success("修改完成");
        } else {
            return error("异常");
        }
    }

    /**
     * 查询数据
     *
     * @param tableName
     * @return
     */
    @PostMapping("selectShow")
    public Result<List<TableShow>> selectShow(@RequestParam String tableName) {
        List<TableShow> list = service.selectShow(tableName);
        return success(list);
    }

    /**
     * 查看整体
     *
     * @return
     */
    @PostMapping("selTotalNum")
    public Result<ToTalNum> selTotalNum() {
        ToTalNum toTalNum = service.selTotalNum();
        return success(toTalNum);
    }


}
