package com.muyu.system.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.system.domain.AssetModel;
import com.muyu.system.domain.Library;
import com.muyu.system.domain.vo.TableVo;
import com.muyu.system.service.IAssetModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 资产详情Controller
 *
 * @author muyu
 * @date 2024-04-21
 */
@RestController
@RequestMapping("/model")
public class AssetModelController extends BaseController {
    @Autowired
    private IAssetModelService assetModelService;

    /**
     * 查询资产详情列表
     */
    @RequiresPermissions("system:model:list")
    @GetMapping("/list")
    public Result<TableDataInfo<AssetModel>> list(AssetModel assetModel) {
        startPage();
        List<AssetModel> list = assetModelService.selectAssetModelList(assetModel);
        return getDataTable(list);
    }

    /**
     * 导出资产详情列表
     */
    @RequiresPermissions("system:model:export")
    @Log(title = "资产详情", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AssetModel assetModel) {
        List<AssetModel> list = assetModelService.selectAssetModelList(assetModel);
        ExcelUtil<AssetModel> util = new ExcelUtil<AssetModel>(AssetModel.class);
        util.exportExcel(response, list, "资产详情数据");
    }

    /**
     * 获取资产详情详细信息
     */
    @RequiresPermissions("system:model:query")
    @GetMapping(value = "/{annotation}")
    public Result getInfo(@PathVariable("annotation") String annotation) {
        return success(assetModelService.selectAssetModelByAnnotation(annotation));
    }

    /**
     * 新增资产详情
     */
    @RequiresPermissions("system:model:add")
    @Log(title = "资产详情", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody AssetModel assetModel) {
        return toAjax(assetModelService.insertAssetModel(assetModel));
    }

    /**
     * 修改资产详情
     */
    @RequiresPermissions("system:model:edit")
    @Log(title = "资产详情", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody AssetModel assetModel) {
        return toAjax(assetModelService.updateAssetModel(assetModel));
    }

    /**
     * 删除资产详情
     */
    @RequiresPermissions("system:model:remove")
    @Log(title = "资产详情", businessType = BusinessType.DELETE)
    @DeleteMapping("/{annotations}")
    public Result remove(@PathVariable String[] annotations) {
        return toAjax(assetModelService.deleteAssetModelByAnnotations(annotations));
    }

    /**
     * 获取数据库的所有表
     *
     * @return
     */
    @GetMapping("library")
    public Result<List<Library>> library() {
        List<Library> list = assetModelService.library();
        return Result.success(list);
    }

    /**
     * 数据详情
     *
     * @param libraryName
     * @return
     */
    @GetMapping("selectTable/{libraryName}")
    public Result<List<TableVo>> selectTable(@PathVariable String libraryName) {
        List<TableVo> list = assetModelService.selectTable(libraryName);
        return Result.success(list);
    }
}
