package com.muyu.system.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.system.domain.DataAccess;
import com.muyu.system.service.IDataAccessService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.sql.SQLException;
import java.util.List;

/**
 * 数据接入Controller
 *
 * @author muyu
 * @date 2024-04-21
 */
@RestController
@RequestMapping("/access")
public class DataAccessController extends BaseController {
    @Autowired
    private IDataAccessService dataAccessService;

    /**
     * 查询数据接入列表
     */
    @RequiresPermissions("system:access:list")
    @GetMapping("/list")
    public Result<TableDataInfo<DataAccess>> list(DataAccess dataAccess) {
        startPage();
        List<DataAccess> list = dataAccessService.selectDataAccessList(dataAccess);
        return getDataTable(list);
    }

    /**
     * 导出数据接入列表
     */
    @RequiresPermissions("system:access:export")
    @Log(title = "数据接入", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataAccess dataAccess) {
        List<DataAccess> list = dataAccessService.selectDataAccessList(dataAccess);
        ExcelUtil<DataAccess> util = new ExcelUtil<DataAccess>(DataAccess.class);
        util.exportExcel(response, list, "数据接入数据");
    }

    /**
     * 获取数据接入详细信息
     */
    @RequiresPermissions("system:access:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id) {
        return success(dataAccessService.selectDataAccessById(id));
    }

    /**
     * 新增数据接入
     */
    @RequiresPermissions("system:access:add")
    @Log(title = "数据接入", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody DataAccess dataAccess) throws SQLException {
        return toAjax(dataAccessService.insertDataAccess(dataAccess));
    }

    /**
     * 修改数据接入
     */
    @RequiresPermissions("system:access:edit")
    @Log(title = "数据接入", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody DataAccess dataAccess) {
        return toAjax(dataAccessService.updateDataAccess(dataAccess));
    }

    /**
     * 删除数据接入
     */
    @RequiresPermissions("system:access:remove")
    @Log(title = "数据接入", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids) {
        return toAjax(dataAccessService.deleteDataAccessByIds(ids));
    }

    /**
     * 获取最后一位数据
     *
     * @return
     */
    @PostMapping("/oneDataAccess")
    public Result<DataAccess> oneDataAccess() {
        DataAccess dataAccess = dataAccessService.oneDataAccess();
        return success(dataAccess);
    }

    /**
     * 同步
     *
     * @param dataAccess
     * @return
     */
    @PostMapping("/synchronization")
    public Result synchronization(@RequestBody DataAccess dataAccess) throws Exception {
        Integer id = dataAccessService.synchronization(dataAccess);
        if (id > 0) {
            return success("成功同步");
        } else {
            return error("同步失败");
        }
    }


}
