package com.muyu.system.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.system.domain.DictData;
import com.muyu.system.domain.DictType;
import com.muyu.system.service.DictService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/dicts")
public class DictController extends BaseController {
    @Autowired
    private DictService service;

    /**
     * 查看字典
     *
     * @return
     */
    @PostMapping("listDictType")
    public Result<List<DictType>> listDictType() {
        List<DictType> list = service.listDictType();
        return success(list);
    }

    /**
     * 查看字典的值
     *
     * @param dictType
     * @return
     */
    @PostMapping("selDictData/{dictType}")
    public Result<List<DictData>> selDictData(@PathVariable String dictType) {
        List<DictData> list = service.selDictData(dictType);
        return success(list);
    }

    /**
     * 新增字典
     */
    @PostMapping("indexDictType/{dictType}")
    public Result indexDictType(@PathVariable String dictType) {
        Integer id = service.indexDictType(dictType);
        if (id > 0) {
            return success("字典添加成功");
        } else {
            return error("字典添加失败");
        }
    }

    @PostMapping("selectDictData")
    public Result<DictData> selectDictData(@RequestBody DictData dictData) {
        DictData dictData1 = service.selectDictData(dictData);
        return success(dictData1);
    }

    /**
     * 字典字段添加
     *
     * @param dictData
     * @return
     */
    @PostMapping("indexDictData")
    public Result indexDictData(@RequestBody DictData dictData) {
        Integer id = service.indexDictData(dictData);
        if (id > 0) {
            return success("成功");
        } else {
            return error("失败");
        }
    }

    /**
     * 删除字典属性
     *
     * @param dictData
     * @return
     */
    @PostMapping("deleteDictData")
    public Result deleteDictData(@RequestBody DictData dictData) {
        Integer id = service.deleteDictData(dictData);
        if (id > 0) {
            return success("成功");
        } else {
            return error("失败");
        }
    }

    /**
     * 删除字典
     *
     * @param dictType
     * @return
     */
    @PostMapping("deleteDictType")
    public Result deleteDictType(@RequestParam String dictType) {
        Integer id;
        Integer i1 = service.deleteDictType(dictType);
        Integer i2 = service.deleteDictDatas(dictType);
        if (i1 > 0 && i2 > 0) {
            id = 1;
        } else {
            id = 0;
        }
        if (id > 0) {
            return success("成功");
        } else {
            return error("失败");
        }
    }

    /**
     * 查看详情字典
     *
     * @param dictType
     * @return
     */
    @PostMapping("selectDictType")
    public Result<DictType> selectDictType(@RequestParam String dictType) {
        DictType type = service.selectDictType(dictType);
        List<DictData> list = service.selDictData(dictType);
        type.setDictDataList(list);
        return success(type);
    }

}
