package com.muyu.system.service.impl;

import com.muyu.system.domain.AssetModel;
import com.muyu.system.domain.Library;
import com.muyu.system.domain.vo.TableVo;
import com.muyu.system.mapper.AssetModelMapper;
import com.muyu.system.service.IAssetModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 资产详情Service业务层处理
 *
 * @author muyu
 * @date 2024-04-21
 */
@Service
public class AssetModelServiceImpl implements IAssetModelService {
    @Autowired
    private AssetModelMapper assetModelMapper;

    /**
     * 查询资产详情
     *
     * @param annotation 资产详情主键
     * @return 资产详情
     */
    @Override
    public AssetModel selectAssetModelByAnnotation(String annotation) {
        return assetModelMapper.selectAssetModelByAnnotation(annotation);
    }

    /**
     * 查询资产详情列表
     *
     * @param assetModel 资产详情
     * @return 资产详情
     */
    @Override
    public List<AssetModel> selectAssetModelList(AssetModel assetModel) {
        return assetModelMapper.selectAssetModelList(assetModel);
    }

    /**
     * 新增资产详情
     *
     * @param assetModel 资产详情
     * @return 结果
     */
    @Override
    public int insertAssetModel(AssetModel assetModel) {
        return assetModelMapper.insertAssetModel(assetModel);
    }

    /**
     * 修改资产详情
     *
     * @param assetModel 资产详情
     * @return 结果
     */
    @Override
    public int updateAssetModel(AssetModel assetModel) {
        return assetModelMapper.updateAssetModel(assetModel);
    }

    /**
     * 批量删除资产详情
     *
     * @param annotations 需要删除的资产详情主键
     * @return 结果
     */
    @Override
    public int deleteAssetModelByAnnotations(String[] annotations) {
        return assetModelMapper.deleteAssetModelByAnnotations(annotations);
    }

    /**
     * 删除资产详情信息
     *
     * @param annotation 资产详情主键
     * @return 结果
     */
    @Override
    public int deleteAssetModelByAnnotation(String annotation) {
        return assetModelMapper.deleteAssetModelByAnnotation(annotation);
    }

    @Override
    public List<Library> library() {
        List<Library> list2 = assetModelMapper.library2();
        System.out.println(list2);
        return list2;
    }

    @Override
    public List<TableVo> selectTable(String libraryName) {
        String database = assetModelMapper.database();
        List<TableVo> list = assetModelMapper.selectTable(database, libraryName);
        System.out.println(list);
        return list;
    }
}
