package com.muyu.system.service.impl;

import com.muyu.common.system.domain.SysDept;
import com.muyu.system.domain.DataAccess;
import com.muyu.system.domain.ListStructure;
import com.muyu.system.domain.vo.SysUserVo;
import com.muyu.system.mapper.AccreditMapper;
import com.muyu.system.mapper.DataAccessMapper;
import com.muyu.system.mapper.SysDeptMapper;
import com.muyu.system.mapper.SysUserMapper;
import com.muyu.system.service.AccreditService;
import com.muyu.system.service.IDataAccessService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


/**
 * 数据接入Service业务层处理
 *
 * @author muyu
 * @date 2024-04-21
 */
@Service
public class DataAccessServiceImpl implements IDataAccessService {
    @Autowired
    private DataAccessMapper dataAccessMapper;
    @Autowired
    private SysDeptMapper sysDeptMapper;
    @Autowired
    private AccreditService accreditService;
    @Autowired
    private AccreditMapper accreditMapper;
    @Autowired
    private SysUserMapper sysUserMapper;


    /**
     * 查询数据接入
     *
     * @param id 数据接入主键
     * @return 数据接入
     */
    @Override
    public DataAccess selectDataAccessById(Long id) {
        return dataAccessMapper.selectDataAccessById(id);
    }

    /**
     * 查询数据接入列表
     *
     * @param dataAccess 数据接入
     * @return 数据接入
     */
    @Override
    public List<DataAccess> selectDataAccessList(DataAccess dataAccess) {
        return dataAccessMapper.selectDataAccessList(dataAccess);
    }

    /**
     * 新增数据接入
     *
     * @param dataAccess 数据接入
     * @return 结果
     */
    @Override
    public int insertDataAccess(DataAccess dataAccess) throws SQLException {
        int i = dataAccessMapper.insertDataAccess(dataAccess);
        return i;
    }

    /**
     * 修改数据接入
     *
     * @param dataAccess 数据接入
     * @return 结果
     */
    @Override
    public int updateDataAccess(DataAccess dataAccess) {
        return dataAccessMapper.updateDataAccess(dataAccess);
    }

    /**
     * 批量删除数据接入
     *
     * @param ids 需要删除的数据接入主键
     * @return 结果
     */
    @Override
    public int deleteDataAccessByIds(Long[] ids) {
        return dataAccessMapper.deleteDataAccessByIds(ids);
    }

    /**
     * 删除数据接入信息
     *
     * @param id 数据接入主键
     * @return 结果
     */
    @Override
    public int deleteDataAccessById(Long id) {
        return dataAccessMapper.deleteDataAccessById(id);
    }

    /**
     * 获取最后一位数据
     *
     * @return
     */
    @Override
    public DataAccess oneDataAccess() {
        return dataAccessMapper.oneDataAccess();
    }

    @Override
    public Integer synchronization(DataAccess dataAccess) throws Exception {
        Integer id = 0;
        List<String> stringList = new ArrayList<>();
        // 连接数据
//        Class<?> aClass = Class.forName("com.mysql.cj.jdbc.Driver");
//        Driver driver = (Driver) aClass.newInstance();
////            //获取数据库连接
//        Connection connection = DriverManager.getConnection("jdbc:mysql://" + dataAccess.getHost() + ":"+ dataAccess.getPort() +"/"+ dataAccess.getDatabaseName() +"?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8",dataAccess.getUsername(),dataAccess.getPassword());
////            //获取数据库名
////            //获取数据库的元数据
//        DatabaseMetaData metaData = connection.getMetaData();
//        //获取全部表名
//        Statement statement = connection.createStatement();
        //开启多线程池
//        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(10, 50, 10, TimeUnit.MINUTES, new LinkedBlockingDeque<>(10));
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(Integer.valueOf(dataAccess.getInitNum().toString()), Integer.valueOf(dataAccess.getMaxNum().toString()), Integer.valueOf(dataAccess.getMaxWaitTime().toString()), TimeUnit.SECONDS, new LinkedBlockingDeque<>(Integer.valueOf(dataAccess.getMaxWaitSize().toString())));

        //第一个 树形表
        CompletableFuture<Integer> completable1 = CompletableFuture.supplyAsync(() -> {
            //先清除同数据库叶子表
            Integer ini = dataAccessMapper.selectAccessMapper(dataAccess);
            if (ini > 0) {
                dataAccessMapper.deleteTableList(dataAccess);
            }
            Integer num = 0;
            Integer i = 0;
            try {
                // 连接数据
                Class<?> aClass = Class.forName("com.mysql.cj.jdbc.Driver");
                Driver driver = (Driver) aClass.newInstance();
//            //获取数据库连接
                Connection connection = DriverManager.getConnection("jdbc:mysql://" + dataAccess.getHost() + ":" + dataAccess.getPort() + "/" + dataAccess.getDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8", dataAccess.getUsername(), dataAccess.getPassword());
//            //获取数据库名
//            //获取数据库的元数据
                DatabaseMetaData metaData = connection.getMetaData();
                //获取全部表名
                Statement statement = connection.createStatement();
                ResultSet rst = statement.executeQuery("select TABLE_NAME, TABLE_COMMENT from INFORMATION_SCHEMA.Tables where table_schema = '" + dataAccess.getDatabaseName() + "'");
                while (rst.next()) {
                    PreparedStatement preparedStatement = connection.prepareStatement("select count(*) from " + rst.getString(1));
                    ResultSet rls = preparedStatement.executeQuery();
                    while (rls.next()) {
                        int anInt = rls.getInt(1);
                        // 叶子类增加
                        num += anInt;
                        i++;
                        dataAccessMapper.inserttableList(dataAccess, rst.getString(1), rst.getString(2), anInt);
                        System.out.println("表名" + rst.getString(1) + ",注释:" + rst.getString(2) + "数量：" + anInt);
                    }
                }
                // 处理树表
                dataAccessMapper.deleteFrimary(dataAccess);
                dataAccessMapper.indexFrimary(dataAccess, num, i);
                rst.close();
            } catch (Exception e) {
                return 0;
            }
            return 1;
        }, threadPoolExecutor);
        //第二个 部门中间表 用户中间表
        CompletableFuture<Integer> completable2 = CompletableFuture.supplyAsync(() -> {
            //获取全部表名
            ResultSet rs = null;
            try {
                // 连接数据
                Class<?> aClass = Class.forName("com.mysql.cj.jdbc.Driver");
                Driver driver = (Driver) aClass.newInstance();
//            //获取数据库连接
                Connection connection = DriverManager.getConnection("jdbc:mysql://" + dataAccess.getHost() + ":" + dataAccess.getPort() + "/" + dataAccess.getDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8", dataAccess.getUsername(), dataAccess.getPassword());
//            //获取数据库名
//            //获取数据库的元数据
                DatabaseMetaData metaData = connection.getMetaData();
                //获取全部表名
                Statement statement = connection.createStatement();
                rs = statement.executeQuery("SHOW TABLES");
                //获取所有部门
                List<SysDept> list = sysDeptMapper.listDeptList();
                //获取所有用户
                List<SysUserVo> sysUserVos = sysUserMapper.listSelectSysUser();
                while (rs.next()) {
                    /**
                     *部门
                     */
                    for (SysDept sysDept : list) {
                        Integer sysDeptIds = dataAccessMapper.selectOneMiddel(sysDept.getDeptId(), dataAccess.getId(), dataAccess.getDatabaseName(), rs.getString(1));
                        if (sysDeptIds == 0) {
                            //添加中间件
                            dataAccessMapper.insertMiddle(sysDept.getDeptId(), dataAccess.getId(), dataAccess.getDatabaseName(), rs.getString(1));
                        }
                    }
                    //用户
                    for (SysUserVo sysUserVo : sysUserVos) {
                        Integer sysUserVoIds = dataAccessMapper.selectOneUserMiddel(sysUserVo.getUserId(), dataAccess.getId(), dataAccess.getDatabaseName(), rs.getString(1));
                        if (sysUserVoIds == 0) {
                            //添加中间件
                            dataAccessMapper.insertUserMiddle(sysUserVo.getUserId(), dataAccess.getId(), dataAccess.getDatabaseName(), rs.getString(1));
                        }
                    }
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
            //获取字段名
            return 1;
        }, threadPoolExecutor);
        //第三个
        CompletableFuture<Integer> completable3 = CompletableFuture.supplyAsync(() -> {
            try {
                // 连接数据
                Class<?> aClass = Class.forName("com.mysql.cj.jdbc.Driver");
                Driver driver = (Driver) aClass.newInstance();
//            //获取数据库连接
                Connection connection = DriverManager.getConnection("jdbc:mysql://" + dataAccess.getHost() + ":" + dataAccess.getPort() + "/" + dataAccess.getDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8", dataAccess.getUsername(), dataAccess.getPassword());
//            //获取数据库名
//            //获取数据库的元数据
                DatabaseMetaData metaData = connection.getMetaData();
                //获取全部表名
                Statement statement = connection.createStatement();
                //获取全部表名
                ResultSet rs = statement.executeQuery("SHOW TABLES");

                //获取字段名
                while (rs.next()) {
                    stringList.add(rs.getString(1));
                }
                //处理表名
                for (String s : stringList) {
                    int i1 = accreditMapper.selectOneSqlJdbc(dataAccess.getDatabaseName(), s);
                    if (i1 > 0) {
                    } else {
                        PreparedStatement pst = connection.prepareCall("select * from " + s);
                        ResultSetMetaData rsd = pst.getMetaData();
                        //获取元数据
                        Map<String, String> columnComments = getColumnComments(metaData, s);
                        //添加数据结构
                        for (int i = 0; i < rsd.getColumnCount(); i++) {
                            System.out.println("java类型" + rsd.getColumnClassName(i + 1));
                            System.out.println("数据库类型" + rsd.getColumnTypeName(i + 1));
                            System.out.println("字段名称" + rsd.getColumnName(i + 1));
                            System.out.println("表名" + rsd.getTableName(i + 1));
                            System.out.println("数据库名" + rsd.getCatalogName(i + 1));
                            System.out.println("模式名" + rsd.getSchemaName(i + 1));
                            System.out.println("列数" + rsd.getColumnCount());
                            System.out.println("列类型" + rsd.getColumnType(i + 1));
                            System.out.println("列标签" + rsd.getColumnLabel(i + 1));
                            System.out.println("列显示大小" + rsd.getColumnDisplaySize(i + 1));
                            System.out.println("精度" + rsd.getPrecision(i + 1));
                            System.out.println("规模" + rsd.getScale(i + 1));
                            System.out.println("自增" + rsd.isAutoIncrement(i + 1));
                            System.out.println("是否为空" + rsd.isNullable(i + 1));
                            System.out.println("区分大小写" + rsd.isCaseSensitive(i + 1));
                            System.out.println("是否是货币" + rsd.isCurrency(i + 1));
                            System.out.println("可搜索" + rsd.isSearchable(i + 1));
                            System.out.println("签署" + rsd.isSigned(i + 1));
                            System.out.println("绝对可以写" + rsd.isDefinitelyWritable(i + 1));
                            System.out.println("只读" + rsd.isReadOnly(i + 1));
                            System.out.println("是可写的" + rsd.isWritable(i + 1));
                            System.out.println("注解" + columnComments.get(rsd.getColumnName(i + 1)));
                            ListStructure listStructure = new ListStructure();
                            //获取数据库名
                            listStructure.setCatalogName(rsd.getCatalogName(i + 1));
                            //获取表名
                            listStructure.setTableName(rsd.getTableName(i + 1));

                            //获取映射类型
                            listStructure.setColumnTypeName(rsd.getColumnTypeName(i + 1));
                            //获取字段名称
                            listStructure.setColumnName(rsd.getColumnName(i + 1));
                            //获取长度
                            listStructure.setPrecisions(rsd.getPrecision(i + 1));
                            //获取小数
                            listStructure.setScale(rsd.getScale(i + 1));
                            //获取非空
                            listStructure.setIsNull(rsd.isNullable(i + 1));
                            //获取默认
                            listStructure.setSchemaName(rsd.getSchemaName(i + 1));
                            //获取注解
                            listStructure.setColumnComments(columnComments.get(rsd.getColumnName(i + 1)));
                            accreditMapper.insertListStructure(listStructure);
                        }
                    }
                }
            } catch (Exception e) {
                return 0;
            }
            return 1;
        }, threadPoolExecutor);

        CompletableFuture<Integer> completable4 = CompletableFuture.supplyAsync(() -> {
            // 连接数据
            try {
//            //获取数据库连接
                Connection connection = DriverManager.getConnection("jdbc:mysql://" + dataAccess.getHost() + ":" + dataAccess.getPort() + "/" + dataAccess.getDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8", dataAccess.getUsername(), dataAccess.getPassword());
//            //获取数据库名
//            //获取数据库的元数据
                DatabaseMetaData metaData = connection.getMetaData();
                //获取全部表名
                Statement statement = connection.createStatement();
                ResultSet rst = statement.executeQuery("select TABLE_NAME from INFORMATION_SCHEMA.Tables where table_schema = '" + dataAccess.getDatabaseName() + "'");
                accreditMapper.deleteTableShow(dataAccess.getName());
                //所有表名
                ArrayList<String> strings = new ArrayList<>();
                while (rst.next()) {
                    strings.add(rst.getString(1));
                    System.out.println(rst.getString(1));
                }
                System.out.println(strings);
                for (String string : strings) {
                    // 执行查询
                    ResultSet rs = statement.executeQuery("select * from " + string);
                    // 获取查询的结果集
                    ResultSetMetaData md = rs.getMetaData();
                    System.out.println(md);
                    int columnCount = md.getColumnCount();
                    System.out.println(columnCount);
                    //获取结果集的元数据
                    //遍历结果集
                    while (rs.next()) {
                        //遍历每一列
                        for (int i = 1; i <= columnCount; i++) {
                            // 获取列名
                            String columnName = md.getColumnName(i);
                            System.out.println(columnName);
                            //获取表名
                            String tableName = md.getTableName(i);
                            System.out.println(tableName);
                            //字段值
                            Object val = rs.getObject(i);
                            System.out.println(val);
                            //字段类型
                            String columnClassName = md.getColumnClassName(i);
                            accreditMapper.indexTableShow(dataAccess.getName() + "." + tableName + "." + columnName, columnClassName, val.toString());
                        }
                    }
                }
            } catch (Exception e) {
                return 0;
            }
            return 1;
        }, threadPoolExecutor);

        System.out.println("completable1:" + completable1.join());
        System.out.println("completable2:" + completable2.join());
        System.out.println("completable3:" + completable3.join());
        System.out.println("completable4:" + completable4.join());
        if (completable1.join() + completable2.join() + completable3.join() + completable4.join() == 4) {
            id = 1;
        } else {
            id = 0;
        }
        return id;
    }

    //获取字段注解
    private Map<String, String> getColumnComments(DatabaseMetaData metaData, String typeName) {
        Map<String, String> columnComents = new HashMap<>();
        try (ResultSet colums = metaData.getColumns(null, null, typeName, null)) {
            while (colums.next()) {
                String columsName = colums.getString("COLUMN_NAME");
                String columsComment = colums.getString("REMARKS");
                columnComents.put(columsName, columsComment);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return columnComents;
    }
}
